// Copyright (C) 2020 Intel Corporation

// SPDX-License-Identifier: MIT

#include <CL/sycl.hpp>
#include <array>
using namespace sycl;
constexpr int N = 4;

int main() {
  std::array<int,N> a, b, c;
  for (int i = 0; i < N; i++) {
    a[i] = i+100;
    b[i] = i+200;
    c[i] = i+300;
  }

  queue 
  Q{cpu_selector{}},
  Q2{cpu_selector{}},
  Q3{cpu_selector{}},
  Q4{cpu_selector{}};

  // We will learn how to simplify this example later
  buffer A{a};
  buffer B{b};
  buffer C{c};

  Q.submit([&](handler &h) {
      accessor accA(A, h, read_only);
      accessor accB(B, h, write_only);
      h.parallel_for( // computeB
        N,
        [=](id<1> i) { 
          accB[i] = accA[i] + 2000; 
          
        });
    });

  Q2.submit([&](handler &h) {
      accessor accA2(A, h, read_only);
      h.parallel_for( // readA
        N,
        [=](id<1> i) {
          // Useful only as an example
          int data = accA2[i];
        });
    });

  Q4.submit([&](handler &h) {
      accessor accA4(A, h, read_only);
      h.parallel_for( // readA
        N,
        [=](id<1> i) {
          // Useful only as an example
          int data = accA4[i];
        });
    });

  Q3.submit([&](handler &h) {
      // RAW of buffer B
      accessor accB3(B, h, read_only);
      accessor accC3(C, h, write_only);
      h.parallel_for( // computeC
        N,
        [=](id<1> i) { 
          accC3[i] = accB3[i] + 30000; 
          });
    });

  host_accessor host_accCrw(C, read_write);
  for (int i = 0; i < N; i++) {
    host_accCrw[i] = host_accCrw[i] +100000;
  }

  // read C on host
  host_accessor host_accC(C, read_only);
  for (int i = 0; i < N; i++) {
    std::cout << host_accC[i] << "\n";
  }
  std::cout << "\n";
  return 0;
}
